@@ -1,7 +1,7 @@
 /*
  * The XML and API file generator module for SIP.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -57,8 +57,9 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
 static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp);
 static void xmlIndent(int indent, FILE *fp);
 static const char *dirAttribute(argDef *ad);
-static void exportDefaultValue(argDef *ad, int in_str, FILE *fp);
 static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope);
+static int exportPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd,
+        int sec, int names, int defaults, int in_str, int is_signal);
 
 
 /*
@@ -239,8 +240,8 @@ static int apiOverload(sipSpec *pt, moduleDef *mod, classDef *scope,
     prScopedPythonName(fp, scope, od->common->pyname->text);
     fprintf(fp, "?%d", METHOD_ID);
 
-    need_sec = prPythonSignature(pt, fp, &od->pysig, sec, TRUE, TRUE, FALSE,
-            FALSE);
+    need_sec = exportPythonSignature(pt, fp, &od->pysig, sec, TRUE, TRUE,
+            FALSE, FALSE);
 
     fprintf(fp, "\n");
 
@@ -281,9 +282,7 @@ static int apiArgument(sipSpec *pt, argDef *ad, int out, int need_comma,
             fprintf(fp, " %s", ad->name->text);
 
         fprintf(fp, "=");
-        prcode(fp, "%M");
-        exportDefaultValue(ad, in_str, fp);
-        prcode(fp, "%M");
+        prDefaultValue(ad, in_str, fp);
     }
 
     return TRUE;
@@ -355,45 +354,45 @@ static void xmlClass(sipSpec *pt, moduleDef *mod, classDef *cd, FILE *fp)
         return;
     }
 
-    xmlIndent(indent++, fp);
-    fprintf(fp, "<Class name=\"");
-    prScopedPythonName(fp, cd->ecd, cd->pyname->text);
-    fprintf(fp, "\"");
-
-    if (cd->picklecode != NULL)
-        fprintf(fp, " pickle=\"1\"");
-
-    if (cd->convtocode != NULL)
-        fprintf(fp, " convert=\"1\"");
-
-    if (cd->convfromcode != NULL)
-        fprintf(fp, " convertfrom=\"1\"");
-
-    if (cd->real != NULL)
-        fprintf(fp, " extends=\"%s\"", cd->real->iff->module->name);
-
-    if (cd->supers != NULL)
+    if (!isHiddenNamespace(cd))
     {
-        classList *cl;
+        xmlIndent(indent++, fp);
+        fprintf(fp, "<Class name=\"");
+        prScopedPythonName(fp, cd->ecd, cd->pyname->text);
+        fprintf(fp, "\"");
 
-        fprintf(fp, " inherits=\"");
+        if (cd->picklecode != NULL)
+            fprintf(fp, " pickle=\"1\"");
 
-        for (cl = cd->supers; cl != NULL; cl = cl->next)
+        if (cd->convtocode != NULL)
+            fprintf(fp, " convert=\"1\"");
+
+        if (cd->convfromcode != NULL)
+            fprintf(fp, " convertfrom=\"1\"");
+
+        if (cd->real != NULL)
+            fprintf(fp, " extends=\"%s\"", cd->real->iff->module->name);
+
+        if (cd->supers != NULL)
         {
-            if (cl != cd->supers)
-                fprintf(fp, " ");
+            classList *cl;
 
-            prScopedPythonName(fp, cl->cd->ecd, cl->cd->pyname->text);
+            fprintf(fp, " inherits=\"");
+
+            for (cl = cd->supers; cl != NULL; cl = cl->next)
+            {
+                if (cl != cd->supers)
+                    fprintf(fp, " ");
+
+                prScopedPythonName(fp, cl->cd->ecd, cl->cd->pyname->text);
+            }
+
+            fprintf(fp, "\"");
         }
 
-        fprintf(fp, "\"");
+        fprintf(fp, ">\n");
     }
 
-    fprintf(fp, ">\n");
-
-    xmlEnums(pt, mod, cd, indent, fp);
-    xmlVars(pt, mod, cd, indent, fp);
-
     for (ct = cd->ctors; ct != NULL; ct = ct->next)
     {
         if (isPrivateCtor(ct))
@@ -403,11 +402,17 @@ static void xmlClass(sipSpec *pt, moduleDef *mod, classDef *cd, FILE *fp)
             xmlCtor(pt, cd, ct, TRUE, indent, fp);
     }
 
+    xmlEnums(pt, mod, cd, indent, fp);
+    xmlVars(pt, mod, cd, indent, fp);
+
     for (md = cd->members; md != NULL; md = md->next)
         xmlFunction(pt, cd, md, cd->overs, indent, fp);
 
-    xmlIndent(--indent, fp);
-    fprintf(fp, "</Class>\n");
+    if (!isHiddenNamespace(cd))
+    {
+        xmlIndent(--indent, fp);
+        fprintf(fp, "</Class>\n");
+    }
 }
 
 
@@ -612,6 +617,11 @@ static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
         fprintf(fp, "\"");
     }
 
+    if (isVirtual(od))
+    {
+        fprintf(fp, " virtual=\"1\"");
+    }
+
     if (xtnds != NULL)
     {
         fprintf(fp, " extends=\"");
@@ -707,6 +717,9 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
     if (isAllowNone(ad))
         fprintf(fp, " allownone=\"1\"");
 
+    if (isDisallowNone(ad))
+        fprintf(fp, " disallownone=\"1\"");
+
     if (isTransferred(ad))
         fprintf(fp, " transfer=\"to\"");
     else if (isThisTransferred(ad))
@@ -720,9 +733,9 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
      */
     if (ad->defval && (dir == NULL || strcmp(dir, "out") != 0))
     {
-        prcode(fp, " default=\"%M");
-        exportDefaultValue(ad, FALSE, fp);
-        prcode(fp, "%M\"");
+        prcode(fp, " default=\"");
+        prDefaultValue(ad, FALSE, fp);
+        prcode(fp, "\"");
     }
 
     fprintf(fp, "/>\n");
@@ -783,6 +796,10 @@ static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp)
     case mapped_type:
         type_type = "mappedtype";
         break;
+
+    /* Suppress a compiler warning. */
+    default:
+        ;
     }
 
     if ((type_name = pyType(pt, ad, sec, &type_scope)) != NULL)
@@ -808,38 +825,6 @@ static void xmlIndent(int indent, FILE *fp)
 }
 
 
-/*
- * Export the default value of an argument.
- */
-static void exportDefaultValue(argDef *ad, int in_str, FILE *fp)
-{
-    /* Use any explicitly provided documentation. */
-    if (ad->docval != NULL)
-    {
-        prcode(fp, "%s", ad->docval);
-        return;
-    }
-
-    /* Translate some special cases. */
-    if (ad->defval->next == NULL && ad->defval->vtype == numeric_value)
-    {
-        if (ad->nrderefs > 0 && ad->defval->u.vnum == 0)
-        {
-            prcode(fp, "None");
-            return;
-        }
-
-        if (ad->atype == bool_type || ad->atype == cbool_type)
-        {
-            prcode(fp, ad->defval->u.vnum ? "True" : "False");
-            return;
-        }
-    }
-
-    generateExpression(ad->defval, in_str, fp);
-}
-
-
 /*
  * Get the Python representation of a type.
  */
@@ -984,13 +969,17 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
         break;
 
     case ustring_type:
+        /* Correct for Python v3. */
+        type_name = "bytes";
+        break;
+
     case string_type:
     case sstring_type:
     case wstring_type:
     case ascii_string_type:
     case latin1_string_type:
     case utf8_string_type:
-        type_name = "str";
+        type_name = isArray(ad) ? "bytes" : "str";
         break;
 
     case byte_type:
@@ -1005,6 +994,7 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
     case short_type:
     case int_type:
     case cint_type:
+    case ssize_type:
         type_name = "int";
         break;
 
@@ -1069,27 +1059,11 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
 }
 
 
-/*
- * Generate a scoped Python name.
- */
-void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname)
-{
-    if (scope != NULL)
-    {
-        prScopedPythonName(fp, scope->ecd, NULL);
-        fprintf(fp, "%s.", scope->pyname->text);
-    }
-
-    if (pyname != NULL)
-        fprintf(fp, "%s", pyname);
-}
-
-
 /*
  * Generate a Python signature.
  */
-int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
-        int names, int defaults, int in_str, int is_signal)
+static int exportPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd,
+        int sec, int names, int defaults, int in_str, int is_signal)
 {
     int need_sec = FALSE, need_comma = FALSE, is_res, nr_out, a;
 
