@@ -96,6 +96,27 @@ PyObject *sip_api_convert_from_const_void_ptr_and_size(const void *val,
         SIP_SSIZE_T size);
 
 
+/*
+ * Support for int convertors.
+ */
+PyObject *sipEnableOverflowChecking(PyObject *self, PyObject *args);
+int sip_api_enable_overflow_checking(int enable);
+int sip_api_convert_to_bool(PyObject *o);
+char sip_api_long_as_char(PyObject *o);
+signed char sip_api_long_as_signed_char(PyObject *o);
+unsigned char sip_api_long_as_unsigned_char(PyObject *o);
+short sip_api_long_as_short(PyObject *o);
+unsigned short sip_api_long_as_unsigned_short(PyObject *o);
+int sip_api_long_as_int(PyObject *o);
+unsigned int sip_api_long_as_unsigned_int(PyObject *o);
+long sip_api_long_as_long(PyObject *o);
+unsigned long sip_api_long_as_unsigned_long(PyObject *o);
+#if defined(HAVE_LONG_LONG)
+PY_LONG_LONG sip_api_long_as_long_long(PyObject *o);
+unsigned PY_LONG_LONG sip_api_long_as_unsigned_long_long(PyObject *o);
+#endif
+
+
 extern sipQtAPI *sipQtSupport;  /* The Qt support API. */
 extern sipWrapperType sipSimpleWrapper_Type;    /* The simple wrapper type. */
 extern sipTypeDef *sipQObjectType;  /* The QObject type. */
@@ -122,7 +143,6 @@ void sip_api_end_thread(void);
 void *sip_api_force_convert_to_type(PyObject *pyObj, const sipTypeDef *td,
         PyObject *transferObj, int flags, int *statep, int *iserrp);
 void sip_api_free_sipslot(sipSlot *slot);
-unsigned long sip_api_long_as_unsigned_long(PyObject *o);
 int sip_api_same_slot(const sipSlot *sp, PyObject *rxObj, const char *slot);
 PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs);
 PyObject *sip_api_invoke_slot_ex(const sipSlot *slot, PyObject *sigargs,
